# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.


""" Jobs manager class.
"""

from cloudcontrol.common.jobs.helpers import AutoIncrementCounter
from cloudcontrol.common.jobs.job import JobState


class JobsManager(object):

    """ Store created jobs.
    """

    def __init__(self, logger, interface, store):
        self.logger = logger
        self._jobs = {}
        self._interface = interface
        self._store = store
        self._counter = AutoIncrementCounter(store)

        # Load jobs from store:
        for job in self._store.iter_jobs():
            job.manager = self
            self._jobs[job.id] = job
            self._interface.on_job_created(job)
            self._interface.on_job_updated(job)

    #
    # Jobs methods
    #

    def notify_attrs_changes(self, job):
        """ Notify a change in attributes of a job to the manager.
        """
        if not job.system:
            self._interface.on_job_updated(job)
            self._store.update_job(job)

    def get_attachment_file(self, job, name):
        """ Get an attachment file object open with write mode for the specified
            job.
        """
        if not job.system:
            return self._store.get_attachment_file(job.id, name)

    def read_attachment(self, job, name):
        """ Get content of an attachment in a string.
        """
        if not job.system:
            return self._store.read_attachment(job.id, name)

    #
    # Public methods
    #

    def spawn(self, job_class, owner, system=False, batch=None, settings={}):
        """ Spawn a new job.
        """
        job_id = 'job-%s' % self._counter.get()
        job_logger = self.logger.getChild(str(job_id))
        job = JobState(job_logger, job_class, job_id, self, owner,
                       settings=settings, system=system, batch=batch)
        if not system:
            self._store.create_job(job)
            self._interface.on_job_created(job)
        self._jobs[job_id] = job

        job.start()
        self.logger.info('Started new job #%s of class %s', job_id, job_class)
        return job

    def get(self, job_id):
        """ Get a job by its id.
        """
        return self._jobs[job_id]

    def purge(self, job_id):
        """ Purge a done job.
        """
        job = self.get(job_id)
        if job.state != 'done':
            raise Exception('Job is not done')
        self._store.delete_job(job_id)
        del self._jobs[job.id]
        self._interface.on_job_purged(job.id)
