# This file is part of CloudControl.
#
# CloudControl is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# CloudControl is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with CloudControl.  If not, see <http://www.gnu.org/licenses/>.


""" TQL ast classes.
"""

import re
from fnmatch import fnmatch


class TqlAst(object):

    """ The AST root.
    """

    def __init__(self, expression=None):
        self.expression = expression


class Filter(object):

    """ A filter.
    """

    def __init__(self, name, filter=None, value=None, neg=False):
        self.name = name
        self.filter = filter
        self.value = value
        try:
            self.value = int(value)
        except ValueError:
            try:
                self.value = float(value)
            except ValueError:
                self.is_number = False
            else:
                self.is_number = True
        else:
            self.is_number = True
        self.neg = neg

    def match(self, value):
        if self.is_number:
            # If provided filter value is a number, we must try to cast the tag
            # value as a number to do a value comparison:
            try:
                value = int(value)
            except ValueError:
                try:
                    value = float(value)
                except ValueError:
                    pass

        match = False
        if self.filter == '=' and value == self.value:
            match = True
        elif self.filter == '>' and value > self.value:
            match = True
        elif self.filter == '<' and value < self.value:
            match = True
        elif self.filter == '>=' and value >= self.value:
            match = True
        elif self.filter == '<=' and value <= self.value:
            match = True
        elif self.filter == ':' and fnmatch(str(value), str(self.value)):
            match = True
        elif self.filter == '~' and re.match(str(self.value), str(value)):
            match = True

        if self.neg:
            match = not match

        return match


class FilterPresence(object):

    """ A filter for tag presence in object.
    """

    def __init__(self, name):
        if name.startswith('-'):
            name = name[1:]
            self.invert = True
        else:
            self.invert = False
        self.name = name


class BinaryOperator(object):
    """ Base class for binary operators.
    """

    def __init__(self, left_expression, right_expression):
        self.left_expression = left_expression
        self.right_expression = right_expression


class UnionOperator(BinaryOperator):
    """ An union operator (eg: expr|expr).
    """

    SYMBOL = '|'


class IntersectionOperator(BinaryOperator):
    """ An intersection operator (eg: expr&expr).
    """

    SYMBOL = '&'


class ShowOperator(object):

    """ A show operator (eg: $tag).
    """

    def __init__(self, expression, pattern):
        self.expression = expression
        if pattern.startswith('-'):
            pattern = pattern[1:]
            self.invert = True
        else:
            self.invert = False
        self.pattern = pattern


class SortOperator(object):

    """ A sorting operator (eg: %tag).
    """

    def __init__(self, expression, name):
        self.expression = expression
        if name.startswith('-'):
            name = name[1:]
            self.invert = True
        else:
            self.invert = False
        self.name = name


class LimitOperator(object):

    """ A limitation operator (eg: ^1:100).
    """

    def __init__(self, expression, start=None, stop=None):
        self.expression = expression
        self.start = start
        self.stop = stop
